@@ -1,11 +1,14 @@
-import requests
-from enum import Enum
 import concurrent.futures
 import logging
+import tarfile
+from enum import Enum
 from functools import partial
 from pathlib import Path
-from tqdm import tqdm
+from tempfile import NamedTemporaryFile
+
+import requests
 from gwdc_python import GWDC
+from tqdm import tqdm
 
 from .bilby_job import BilbyJob
 from .file_reference import FileReference, FileReferenceList
@@ -13,6 +16,7 @@ from .utils import rename_dict_keys, convert_dict_keys
 
 GWCLOUD_ENDPOINT = 'https://gwcloud.org.au/bilby/graphql'
 GWCLOUD_FILE_DOWNLOAD_ENDPOINT = 'https://gwcloud.org.au/job/apiv1/file/?fileId='
+GWCLOUD_UPLOADED_JOB_FILE_DOWNLOAD_ENDPOINT = 'https://gwcloud.org.au/bilby/file_download/?fileId='
 
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.INFO)
@@ -133,7 +137,16 @@ class GWCloud:
         return self.get_public_job_list(search="preferred lasky", time_range=TimeRange.ANY)
 
     def _get_job_model_from_query(self, query_data):
-        return BilbyJob(client=self, **rename_dict_keys(query_data, [('id', 'job_id'), ('jobStatus', 'job_status')]))
+        return BilbyJob(
+            client=self,
+            **rename_dict_keys(
+                query_data,
+                [
+                    ('id', 'job_id'),
+                    ('jobStatus', 'job_status'),
+                ]
+            )
+        )
 
     def get_public_job_list(self, search="", time_range=TimeRange.ANY, number=100):
         """Obtains a list of public Bilby jobs, filtering based on the search terms
@@ -252,6 +265,7 @@ class GWCloud:
                         fileSize
                         downloadToken
                     }
+                    isUploadedJob
                 }
             }
         """
@@ -269,10 +283,14 @@ class GWCloud:
             f.pop('isDir')
             file_list.append(FileReference(**convert_dict_keys(f)))
 
-        return file_list
+        return file_list, data['bilbyResultFiles']['isUploadedJob']
 
-    def _get_file_map_fn(self, file_id, file_path, progress_bar):
-        download_url = GWCLOUD_FILE_DOWNLOAD_ENDPOINT + str(file_id)
+    def _get_file_map_fn(self, file_id, file_path, progress_bar, is_uploaded_job=False):
+        endpoint = GWCLOUD_FILE_DOWNLOAD_ENDPOINT \
+            if not is_uploaded_job else \
+            GWCLOUD_UPLOADED_JOB_FILE_DOWNLOAD_ENDPOINT
+
+        download_url = endpoint + str(file_id)
         content = b''
         with requests.get(download_url, stream=True) as request:
             for chunk in request.iter_content(chunk_size=1024 * 16, decode_unicode=True):
@@ -280,7 +298,7 @@ class GWCloud:
                 content += chunk
         return (file_path, content)
 
-    def _get_files_by_reference(self, job_id, file_references):
+    def _get_files_by_reference(self, job_id, file_references, is_uploaded_job=False):
         """Obtains file data when provided a job ID and a FileReferenceList
 
         Parameters
@@ -300,14 +318,26 @@ class GWCloud:
 
         with concurrent.futures.ThreadPoolExecutor(max_workers=20) as executor:
             progress = tqdm(total=file_references.get_total_bytes(), leave=True, unit='B', unit_scale=True)
-            files = list(executor.map(partial(self._get_file_map_fn, progress_bar=progress), file_ids, file_paths))
+            files = list(
+                executor.map(
+                    partial(
+                        self._get_file_map_fn,
+                        progress_bar=progress,
+                        is_uploaded_job=is_uploaded_job
+                    ), file_ids, file_paths
+                )
+            )
             progress.close()
             logger.info(f'All {len(file_ids)} files downloaded!')
 
         return files
 
-    def _save_file_map_fn(self, file_id, file_path, progress_bar):
-        download_url = GWCLOUD_FILE_DOWNLOAD_ENDPOINT + str(file_id)
+    def _save_file_map_fn(self, file_id, file_path, progress_bar, is_uploaded_job=False):
+        endpoint = GWCLOUD_FILE_DOWNLOAD_ENDPOINT \
+            if not is_uploaded_job else \
+            GWCLOUD_UPLOADED_JOB_FILE_DOWNLOAD_ENDPOINT
+
+        download_url = endpoint + str(file_id)
         file_path.parents[0].mkdir(parents=True, exist_ok=True)
 
         with requests.get(download_url, stream=True) as request:
@@ -316,7 +346,8 @@ class GWCloud:
                     progress_bar.update(len(chunk))
                     f.write(chunk)
 
-    def _save_files_by_reference(self, job_id, file_references, root_path, preserve_directory_structure=True):
+    def _save_files_by_reference(self, job_id, file_references, root_path, preserve_directory_structure=True,
+                                 is_uploaded_job=False):
         """Save files when provided a job ID and a FileReferenceList
 
         Parameters
@@ -340,10 +371,18 @@ class GWCloud:
 
         with concurrent.futures.ThreadPoolExecutor(max_workers=20) as executor:
             progress = tqdm(total=file_references.get_total_bytes(), leave=True, unit='B', unit_scale=True)
-            list(executor.map(partial(self._save_file_map_fn, progress_bar=progress), file_ids, file_paths))
+            list(
+                executor.map(
+                    partial(
+                        self._save_file_map_fn,
+                        progress_bar=progress,
+                        is_uploaded_job=is_uploaded_job
+                    ),
+                    file_ids, file_paths
+                )
+            )
             progress.close()
-
-        logger.info(f'All {len(file_ids)} files saved!')
+            logger.info(f'All {len(file_ids)} files saved!')
 
         return 'Success'
 
@@ -399,3 +438,99 @@ class GWCloud:
         data = self.client.request(query=query, variables=variables)
 
         return data['generateFileDownloadIds']['result']
+
+    def _generate_upload_token(self):
+        """Creates a new long lived upload token for use uploading jobs
+
+        Returns
+        -------
+        str
+            The upload token
+        """
+        query = """
+            query GenerateBilbyJobUploadToken {
+                generateBilbyJobUploadToken {
+                  token
+                }
+            }
+        """
+
+        data = self.client.request(query=query)
+        return data['generateBilbyJobUploadToken']['token']
+
+    def upload_job_archive(self, description, job_archive, public=False):
+        """Upload a bilby job to GWCloud by job output archive
+
+        Parameters
+        ----------
+        description : str
+            The description of the job to add to the database
+
+        public : bool
+            If the uploaded job should be public or not
+
+        job_archive : str
+            The path to the job output archive to upload
+
+        Returns
+        -------
+        BilbyJob
+            The created Bilby job
+        """
+        query = """
+            mutation JobUploadMutation($input: UploadBilbyJobMutationInput!) {
+                uploadBilbyJob(input: $input) {
+                    result {
+                        jobId
+                    }
+                }
+            }
+        """
+
+        with open(job_archive, 'rb') as f:
+            variables = {
+                "input": {
+                    "uploadToken": self._generate_upload_token(),
+                    "details": {
+                        "description": description,
+                        "private": not public
+                    },
+                    "jobFile": f
+                }
+            }
+
+            data = self.client.request(query=query, variables=variables, authorize=False)
+
+        job_id = data['uploadBilbyJob']['result']['jobId']
+        return self.get_job_by_id(job_id)
+
+    def upload_job_directory(self, description, job_directory, public=False):
+        """Upload a bilby job to GWCloud by job output directory
+
+        Parameters
+        ----------
+        description : str
+            The description of the job to add to the database
+
+        public : bool
+            If the uploaded job should be public or not
+
+        job_directory : str
+            The path to the job output directory to upload
+
+        Returns
+        -------
+        BilbyJob
+            The created Bilby job
+        """
+
+        # Generate a temporary archive of the job
+        path = Path(job_directory)
+        with NamedTemporaryFile(dir=job_directory, suffix='.tar.gz') as f:
+            with tarfile.open(f.name, "w:gz", compresslevel=2) as tar_handle:
+                for item in path.rglob("*"):
+                    print(item)
+                    tar_handle.add(item, arcname=item.relative_to(path), recursive=False)
+
+            # Upload the archive
+            return self.upload_job_archive(description, f.name, public)
